﻿Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging

Public Class AdobeProgressBar
    Inherits Control

#Region "Properties"
    Private m_Value As Integer = 0
    Private m_Maximum As Integer = 100
    Private m_Style As ProgressBarStyle = ProgressBarStyle.Continuous
    Private m_MarqueeStyle As MarqueeAnimationStyle = AdobeControls.MarqueeAnimationStyle.Wrap
    Private m_MarqueeSpeed As Integer = 1

    ''' <summary>
    ''' Gets or sets the value of the <see cref="AdobeProgressBar" />.
    ''' </summary>
    Public Property Value() As Integer
        Get
            Return m_Value
        End Get
        Set(ByVal value As Integer)
            If value < 0 OrElse value > Me.Maximum Then _
                Throw New ArgumentOutOfRangeException("value")
            m_Value = value
            Me.Invalidate()
            Me.OnValueChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the maximum value the <see cref="AdobeProgressBar" /> will display.
    ''' </summary>
    Public Property Maximum() As Integer
        Get
            Return m_Maximum
        End Get
        Set(ByVal value As Integer)
            If value <= 0 Then _
                Throw New ArgumentOutOfRangeException("value")
            m_Maximum = value
            If Me.Value > Me.Maximum Then _
                Me.Value = Me.Maximum
            Me.Invalidate()
            Me.OnValueChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the <see cref="AdobeProgressBar" />'s display style.
    ''' </summary>
    Public Property Style() As ProgressBarStyle
        Get
            Return m_Style
        End Get
        Set(ByVal value As ProgressBarStyle)
            If value = ProgressBarStyle.Blocks Then _
                value = ProgressBarStyle.Continuous
            m_Style = value
            Me.Invalidate()
            m_IsAnimating = value = ProgressBarStyle.Marquee
            Me.OnProgressBarStyleChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets how the <see cref="AdobeProgressBar" /> will animate when <see cref="Style" /> is set to <see cref="MarqueeAnimationStyle.Wrap" />.
    ''' </summary>
    Public Property MarqueeAnimationStyle() As MarqueeAnimationStyle
        Get
            Return m_MarqueeStyle
        End Get
        Set(ByVal value As MarqueeAnimationStyle)
            m_MarqueeStyle = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets how fast the <see cref="AdobeProgressBar" /> will animate when <see cref="Style" /> is set to <see cref="MarqueeAnimationStyle.Wrap" />.
    ''' </summary>
    Public Property MarqueeAnimationSpeed() As Integer
        Get
            Return m_MarqueeSpeed
        End Get
        Set(ByVal value As Integer)
            m_MarqueeSpeed = value
        End Set
    End Property

    <System.ComponentModel.Browsable(False)> _
    Public Overrides Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
        End Set
    End Property
#End Region

#Region "Constructor"
    Public Sub New()
        Me.DoubleBuffered = True
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint Or _
                    ControlStyles.SupportsTransparentBackColor Or _
                    ControlStyles.OptimizedDoubleBuffer Or ControlStyles.ResizeRedraw, True)
        Me.Height = 34

        If Not Me.DesignMode Then
            Dim t As New Threading.Thread(AddressOf Me.AnimationThread)
            t.IsBackground = True
            t.Start()
        End If
    End Sub
#End Region

#Region "Animation"
    Private m_AnimationFrame As Integer = 0
    Private m_IsAnimating As Boolean = False

    Private Sub AnimationThread()
        Dim inv As New MethodInvoker(AddressOf Me.Invalidate)
        Do Until Me.IsDisposed
            If Me.DesignMode Then Exit Do
            If m_IsAnimating AndAlso Me.Created Then
                If m_AnimationFrame = Integer.MaxValue Then
                    m_AnimationFrame = 0
                Else
                    m_AnimationFrame += 1
                End If
                Me.Invoke(inv)
            End If
            Threading.Thread.Sleep(33)
        Loop
    End Sub
#End Region

#Region "Drawing"
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        g.Clear(AdobeColors.ProgressBarBackgroundColor)
        g.SmoothingMode = SmoothingMode.HighQuality

        'Draw the current progress
        Dim w As Integer = CInt(Me.Width * Me.Value / Me.Maximum), o As Integer = 0
        If Me.Style = ProgressBarStyle.Marquee AndAlso Me.Value < Me.Maximum Then
            Dim f As Integer = CInt((CLng(m_AnimationFrame) * Me.MarqueeAnimationSpeed) Mod Integer.MaxValue)
            Select Case Me.MarqueeAnimationStyle
                Case AdobeControls.MarqueeAnimationStyle.Wrap
                    o = f Mod (Me.Width + w) - w
                Case AdobeControls.MarqueeAnimationStyle.Bounce
                    o = f Mod (Me.Width * 2 - w * 2)
                    If o + w > Me.Width Then _
                        o = Me.Width - (o + w - Me.Width) - w
            End Select
        End If
        If w > 0 Then
            Dim topRect As New Rectangle(o, 0, w, Me.Height \ 2)
            Dim bottomRect As New Rectangle(topRect.X, topRect.Bottom, topRect.Width, topRect.Height)
            Using topBrush As New LinearGradientBrush(topRect, AdobeColors.ProgressBarTopStartColor, AdobeColors.ProgressBarTopEndColor, LinearGradientMode.Vertical)
                g.FillRectangle(topBrush, topRect)
            End Using
            Using bottomBrush As New LinearGradientBrush(bottomRect, AdobeColors.ProgressBarBottomStartColor, AdobeColors.ProgressBarBottomEndColor, LinearGradientMode.Vertical)
                g.FillRectangle(bottomBrush, bottomRect)
            End Using
        End If

        'Draw the borders
        Dim outerBorder As Rectangle = Me.ClientRectangle
        outerBorder.Width -= 1
        outerBorder.Height -= 1
        Dim innerBorder As Rectangle = outerBorder
        innerBorder.Inflate(-1, -1)
        Using outBorderPen As New Pen(AdobeColors.InnerBorderColor)
            g.DrawRectangle(outBorderPen, outerBorder)
        End Using
        Using inBorderPen As New Pen(AdobeColors.BorderColor)
            g.DrawRectangle(inBorderPen, innerBorder)
        End Using
    End Sub
#End Region

#Region "Events"
    Public Event ValueChanged As EventHandler
    Public Event MaximumChanged As EventHandler
    Public Event ProgressBarStyleChanged As EventHandler

    Protected Overridable Sub OnValueChanged(ByVal e As EventArgs)
        RaiseEvent ValueChanged(Me, e)
    End Sub

    Protected Overridable Sub OnMaximumChanged(ByVal e As EventArgs)
        RaiseEvent MaximumChanged(Me, e)
    End Sub

    Protected Overridable Sub OnProgressBarStyleChanged(ByVal e As EventArgs)
        RaiseEvent ProgressBarStyleChanged(Me, e)
    End Sub
#End Region

End Class

Public Enum MarqueeAnimationStyle
    Wrap
    Bounce
End Enum
